from __future__ import annotations

import sys
from pathlib import Path
from typing import NoReturn

from litestar import Litestar, get
from pydantic.dataclasses import dataclass

from satlite import SatlitePlugin, setup_environment
from satlite.application.config.settings import Api, App, Server
from satlite.utils.typed_settings import get_settings


@dataclass
class Settings:
    server: Server
    app: App
    api: Api


def create_app() -> Litestar:
    """Create ASGI application."""

    from litestar import Litestar

    @get('/health')
    def health_check() -> dict[str, str]:
        """Health check endpoint."""
        return {'status': 'ok'}

    return Litestar(plugins=[SatlitePlugin()], route_handlers=[health_check])


settings = get_settings(Settings, prefix='satlite', dotenv_path=Path(__file__).parent / '.env')


def run_cli() -> NoReturn:
    setup_environment(
        app_location='example.__main__:create_app',
        app_settings=settings.app,
        server_settings=settings.server,
    )

    try:
        from litestar.cli.main import litestar_group

        sys.exit(litestar_group())
    except ImportError as exc:
        print(
            'Could not load required libraries. ',
            'Please check your installation and make sure you activated any necessary virtual '
            'environment',
        )
        print(exc)
        sys.exit(1)


if __name__ == '__main__':
    print(settings.server.host, settings.server.port)
    run_cli()



import binascii
import os
from dataclasses import field
from pathlib import Path
from typing import Final

from pydantic import Field
from pydantic.dataclasses import dataclass


@dataclass
class ModuleMeta:
    """Metadata for a module."""

    name: str
    version: str
    summary: str
    base_dir: Path


def get_module_meta(module_name: str) -> ModuleMeta:
    """Get the metadata of a module."""
    from importlib.metadata import metadata

    from litestar.utils.module_loader import module_to_os_path

    BASE_DIR: Final[Path] = module_to_os_path(module_name)
    meta = metadata(module_name)

    return ModuleMeta(
        name=meta.get('name', module_name),
        version=meta.get('version', '0.0.0'),
        summary=meta.get('summary', ''),
        base_dir=BASE_DIR,
    )


@dataclass
class Csrf:
    """Configuration for **CSRF** (**C**ross **S**ite **R**equest **F**orgery) protection."""

    secret: str = field(default=binascii.hexlify(os.urandom(32)).decode(encoding='utf-8'))
    '''A string that is used to create an HMAC to sign the CSRF token.'''

    cookie_name: str = field(default='csrftoken')
    '''The CSRF cookie name.'''

    cookie_path: str = field(default='/')
    '''The CSRF cookie path.'''

    header_name: str = field(default='x-csrftoken')
    '''The header that will be expected in each request.'''

    cookie_secure: bool = field(default=False)
    '''A boolean value indicating whether to set the Secure attribute on the cookie.'''

    cookie_httponly: bool = field(default=False)
    '''A boolean value indicating whether to set the `HttpOnly` attribute on the cookie.'''

    cookie_domain: str | None = field(default=None)
    '''The value to set in the `SameSite` attribute of the cookie.'''

    exclude: str | list[str] | None = field(default=None)
    '''Specifies which hosts can receive the cookie.'''

    exclude_from_csrf_key: str = field(default='exclude_from_csrf')
    '''A set of “safe methods” that can set the cookie.'''

    cookie_samesite: str = field(default='lax')  # "lax", "strict", "none"
    '''A pattern or list of patterns to skip in the CSRF middleware.'''

    safe_methods: set[str] = field(default_factory=lambda: {'GET', 'HEAD', 'OPTIONS'})
    '''An identifier to use on routes to disable CSRF for a particular route.'''


@dataclass
class Cors:
    """Configuration for **CORS** (**C**ross-**O**rigin **R**esource **S**haring)."""

    allowed_origins: list[str] = field(default_factory=lambda: ['*'])
    '''
    List of origins that are allowed. Can use `*` in any component of the path, e.g. `domain.*`.
    Sets the `Access-Control-Allow-Origin` header.
    '''

    allow_methods: list[str] = field(default_factory=lambda: ['*'])
    '''List of allowed HTTP methods. Sets the `Access-Control-Allow-Methods` header.'''

    allow_headers: list[str] = field(default_factory=lambda: ['*'])
    '''List of allowed headers. Sets the `Access-Control-Allow-Headers` header.'''

    allow_credentials: bool = field(default=False)
    '''Boolean dictating whether or not to set the `Access-Control-Allow-Credentials` header.'''

    allow_origin_regex: str | None = field(default=None)
    '''Regex to match origins against.'''

    expose_headers: list[str] = field(default_factory=list)
    '''List of headers that are exposed via the `Access-Control-Expose-Headers` header.'''

    max_age: int = field(default=600)
    '''Response caching TTL in secs, defaults: 600. Sets the `Access-Control-Max-Age` header.'''


@dataclass
class Api:
    cache_expiration: int = field(default=60)
    '''Default cache expiration in secs, when a route handler is configured with `cache=True`.'''

    csrf: Csrf = field(default_factory=Csrf)
    '''Configuration for **CSRF** (**C**ross **S**ite **R**equest **F**orgery) protection.'''

    cors: Cors = field(default_factory=Cors)
    '''Configuration for **CORS** (**C**ross-**O**rigin **R**esource **S**haring).'''


@dataclass
class Server:
    name: str = field(default='satlite')
    '''The name of the HTTP server.'''

    host: str = field(default='127.0.0.1')
    '''The host to bind the Granian server to.'''

    port: int = Field(default=8080, gt=0, le=65535)
    '''The port to bind the Granian server to.'''


@dataclass
class App:
    name: str = field(default='satlite')
    '''The name of the application.'''

    slug: str = field(default='satlite')
    '''A slug for the application (a short, URL-friendly version of the name).'''

    version: str = field(default='0.0.1')
    '''The version of the application.'''

    debug: bool = field(default=False)
    '''A boolean indicating whether to run the application in debug mode.'''

    enable_structlog: bool = field(default=True)
    '''A boolean indicating whether to enable structlog logging.'''


@dataclass
class Vite:
    """Configuration for ViteJS support."""

    asset_url: str = field(default='/static/')
    '''Base URL to generate for static asset references.
    This URL will be prepended to anything generated from Vite.
    '''

    build_command: list[str] = field(default_factory=lambda: ['bun', 'run', 'build'])
    '''Default command to use for building with Vite.'''

    build_watch_command: list[str] = field(default_factory=lambda: ['bun', 'run', 'watch'])
    '''Default command to use for dev building with Vite.'''

    bundle_dir: Path = field(default=Path('web/public'))
    '''Location of the compiled assets from  Vite. The manifest file will also be found here.'''

    detect_nodeenv: bool = field(default=True)
    '''When True, The initializer will install and configure nodeenv if present'''

    dev_mode: bool = field(default=False)
    '''When True, Vite will run with HMR or watch build'''

    host: str = field(default='localhost')
    '''Default host to use for Vite server.'''

    hot_file: str = 'hot'
    '''Name of the hot file.
    This file contains a single line containing the host, protocol, and port the Vite server is
    running.
    '''

    hot_reload: bool = field(default=True)
    '''Enable HMR for Vite development server.'''

    install_command: list[str] = field(default_factory=lambda: ['bun', 'install'])
    '''Default command to use for installing dependencies.'''

    is_react: bool = field(default=True)
    '''Enable React components.'''

    manifest_name: str = field(default='manifest.json')
    '''Name of the manifest file.'''

    port: int = field(default=5173)
    '''Name of the hot file.
    This file contains a single line containing the host, protocol, and port the Vite server is
    running.
    '''

    protocol: str = field(default='http')
    '''Protocol to use for communication'''

    public_dir: Path = field(default=Path('public'))
    '''The optional public directory Vite serves assets from.
    In a standalone Vue or React application, this would be equivalent to the `./public` directory.
    '''

    resource_dir: Path = field(default=Path('resources'))
    '''The directory where all typescript/javascript source are written.
    In a standalone Vue or React application, this would be equivalent to the `./src` directory.
    '''

    root_dir: Path | None = field(default=None)
    '''The base path to your application.
    In a standalone Vue or React application, this would be equivalent to the top-level project
    folder containing the `./src` directory.
    '''

    run_command: list[str] = field(default_factory=lambda: ['bun', 'run', 'dev'])
    '''Default command to use for running Vite.'''

    set_environment: bool = field(default=True)
    '''When True, configuration in this class will be set into environment variables.

    This can be useful to ensure Vite always uses the configuration supplied to the plugin
    '''

    set_static_folders: bool = field(default=True)
    '''When True, Litestar will automatically serve assets at the `ASSET_URL` path.'''

    ssr_enabled: bool = field(default=False)
    '''Enable SSR.'''

    ssr_output_dir: Path | None = field(default=None)
    '''SSR Output path'''

    template_dir: Path = field(default=Path('web/templates'))
    '''The directory jinja templates are stored in.'''

    use_server_lifespan: bool = field(default=True)
    '''Utilize the server lifespan hook to run Vite.'''



from typing import TYPE_CHECKING, Any, cast

from litestar.config.cors import CORSConfig

if TYPE_CHECKING:
    from ..settings import Api as ApiSettings


def default_cors(
    api_settings: 'ApiSettings',
) -> CORSConfig:
    """Default CORS configuration."""
    return CORSConfig(
        allow_origins=api_settings.cors.allowed_origins,
        allow_origin_regex=api_settings.cors.allow_origin_regex,
        allow_methods=cast(Any, api_settings.cors.allow_methods),
        allow_credentials=api_settings.cors.allow_credentials,
        allow_headers=api_settings.cors.allow_headers,
        expose_headers=api_settings.cors.expose_headers,
        max_age=api_settings.cors.max_age,
    )



from typing import TYPE_CHECKING, Any, cast

from litestar.config.csrf import CSRFConfig

if TYPE_CHECKING:
    from ..settings import Api as ApiSettings


def default_csrf(
    api_settings: 'ApiSettings',
) -> CSRFConfig:
    """Default CSRF configuration."""
    return CSRFConfig(
        secret=api_settings.csrf.secret,
        cookie_secure=api_settings.csrf.cookie_secure,
        cookie_name=api_settings.csrf.cookie_name,
        cookie_path=api_settings.csrf.cookie_path,
        header_name=api_settings.csrf.header_name,
        cookie_httponly=api_settings.csrf.cookie_httponly,
        cookie_domain=api_settings.csrf.cookie_domain,
        exclude=api_settings.csrf.exclude,
        exclude_from_csrf_key=api_settings.csrf.exclude_from_csrf_key,
        cookie_samesite=cast(Any, api_settings.csrf.cookie_samesite),
        safe_methods=cast(Any, api_settings.csrf.safe_methods),
    )



from dataclasses import field

import pytest
from pydantic import Field
from pydantic.dataclasses import dataclass
from typed_settings.exceptions import InvalidSettingsError

from satlite.utils.typed_settings import get_settings


@dataclass
class Server:
    host: str = field(default='127.0.0.1')
    port: int = Field(default=8080, gt=0, le=65535)


@dataclass
class Settings:
    server: Server


@pytest.fixture(autouse=True)
def clear_env(monkeypatch):
    # Clear relevant env vars before each test
    monkeypatch.delenv('SATLITE_SERVER_HOST', raising=False)
    monkeypatch.delenv('SATLITE_SERVER_PORT', raising=False)


@pytest.fixture(autouse=True)
def clear_cache():
    # Clear the cache before each test
    get_settings.cache_clear()


def test_get_settings_defaults():
    settings: Settings = get_settings(settings_type=Settings, prefix='satlite', strlist_sep='|')
    assert settings.server.host == '127.0.0.1'
    assert settings.server.port == 8080


def test_get_settings_from_env(monkeypatch):
    monkeypatch.setenv('SATLITE_SERVER_HOST', '127.0.0.1')
    monkeypatch.setenv('SATLITE_SERVER_PORT', '9090')

    settings: Settings = get_settings(settings_type=Settings, prefix='satlite', strlist_sep='|')

    assert settings.server.host == '127.0.0.1'
    assert settings.server.port == 9090


def test_get_settings_invalid_port(monkeypatch):
    monkeypatch.setenv('SATLITE_SERVER_PORT', '70000')  # Above max

    with pytest.raises(InvalidSettingsError):
        get_settings(settings_type=Settings, prefix='satlite', strlist_sep='|')



import logging

import structlog
from litestar.logging import LoggingConfig, StructLoggingConfig
from litestar.logging.config import (
    default_logger_factory,
    default_structlog_standard_lib_processors,
)
from litestar.middleware.logging import LoggingMiddlewareConfig
from litestar.plugins.structlog import StructlogConfig


def default_structlog() -> StructlogConfig:
    return StructlogConfig(
        structlog_logging_config=StructLoggingConfig(
            log_exceptions='debug',
            logger_factory=default_logger_factory(as_json=False),
            standard_lib_logging_config=LoggingConfig(
                root={'level': logging.getLevelName(30), 'handlers': ['queue_listener']},
                formatters={
                    'standard': {
                        '()': structlog.stdlib.ProcessorFormatter,
                        'processors': [*default_structlog_standard_lib_processors(as_json=False)],
                    }
                },
                loggers={
                    '_granian': {
                        'propagate': False,
                        'level': 30,
                        'handlers': ['queue_listener'],
                    },
                    'granian.server': {
                        'propagate': False,
                        'level': 30,
                        'handlers': ['queue_listener'],
                    },
                    'granian.access': {
                        'propagate': False,
                        'level': 30,
                        'handlers': ['queue_listener'],
                    },
                },
            ),
        ),
        middleware_logging_config=LoggingMiddlewareConfig(
            request_log_fields=['path', 'method', 'query', 'path_params'],
            response_log_fields=['status_code', 'body'],
        ),
    )



from typing import Any
from unittest.mock import AsyncMock, MagicMock

import pytest

from satlite.middleware.authentication.inclusive_abstract_auth import (
    InclusiveAbstractAuthenticationMiddleware,
)


# Fixtures
@pytest.fixture
def mock_app() -> AsyncMock:
    """Mock ASGI application."""
    return AsyncMock()


@pytest.fixture
def mock_scope() -> dict:
    """Mock the ASGI scope (request data), including 'route_handler'."""
    route_handler_mock = MagicMock()
    route_handler_mock.opt = {}  # Mock 'opt' as an empty dictionary

    return {
        'type': 'http',
        'path': '',
        'raw_path': b'',
        'method': 'GET',
        'route_handler': route_handler_mock,
    }


@pytest.fixture
def mock_receive() -> MagicMock:
    """Mock the ASGI receive callable."""
    return MagicMock()


@pytest.fixture
def mock_send() -> MagicMock:
    """Mock the ASGI send callable."""
    return MagicMock()


class MockToken:
    """Mock token class."""

    def __init__(self, token: str):
        self.token = token

    @classmethod
    def from_string(cls, token: str) -> 'MockToken':
        return cls(token)


# Tests
@pytest.mark.asyncio
async def test_include_matching(mock_app, mock_scope, mock_receive, mock_send):
    """Test the case when the 'include' pattern matches the request path."""

    include_pattern = '/some/path'

    class TestMiddleware(InclusiveAbstractAuthenticationMiddleware):
        async def authenticate_request(self, connection) -> Any:
            return MagicMock(user='user', auth='auth')

    middleware = TestMiddleware(
        app=mock_app,
        include=include_pattern,
        token_cls=MockToken,
        retrieve_user_handler=AsyncMock(return_value='user'),
    )

    for path in ['/some/path', '/some/path/inner', '/some/path/anything']:
        mock_scope['path'] = path
        mock_scope['raw_path'] = path.encode()

        await middleware(mock_scope, mock_receive, mock_send)
        mock_app.assert_called_once_with(mock_scope, mock_receive, mock_send)
        mock_app.reset_mock()


@pytest.mark.asyncio
async def test_include_and_exclude_matching(mock_app, mock_scope, mock_receive, mock_send):
    """Test include and exclude patterns together and verify auth injection."""

    include_pattern = '/some/path'
    exclude_pattern = '/some/path/but/excluded'

    called_paths = []

    class TestMiddleware(InclusiveAbstractAuthenticationMiddleware):
        async def authenticate_request(self, connection) -> Any:
            called_paths.append(connection.scope['path'])
            return MagicMock(user='user', auth='auth')

    middleware = TestMiddleware(
        app=mock_app,
        include=include_pattern,
        exclude=exclude_pattern,
        token_cls=MockToken,
        retrieve_user_handler=AsyncMock(return_value='user'),
    )

    test_paths = [
        ('/some/path', True),
        ('/some/path/inner', True),
        ('/some/path/but', True),
        ('/some/path/but/excluded', False),
        ('/some/other/path', False),
        ('/foo/bar', False),
    ]

    for path, should_authenticate in test_paths:
        mock_scope['path'] = path
        mock_scope['raw_path'] = path.encode()
        mock_scope.pop('user', None)
        mock_scope.pop('auth', None)

        await middleware(mock_scope, mock_receive, mock_send)

        mock_app.assert_called_once_with(mock_scope, mock_receive, mock_send)
        mock_app.reset_mock()

        if should_authenticate:
            assert mock_scope['user'] == 'user'
            assert mock_scope['auth'] == 'auth'
        else:
            assert 'user' not in mock_scope
            assert 'auth' not in mock_scope

    assert called_paths == [
        '/some/path',
        '/some/path/inner',
        '/some/path/but',
    ]


@pytest.mark.asyncio
async def test_include_and_exclude_matching_sets_user_and_auth(
    mock_app, mock_scope, mock_receive, mock_send
):
    include_pattern = '/some/path'
    exclude_pattern = '/some/path/but/excluded'

    class TestMiddleware(InclusiveAbstractAuthenticationMiddleware):
        async def authenticate_request(self, connection) -> Any:
            return MagicMock(user='the-user', auth='the-auth')

    middleware = TestMiddleware(
        app=mock_app,
        include=include_pattern,
        exclude=exclude_pattern,
        token_cls=MockToken,
        retrieve_user_handler=AsyncMock(return_value='the-user'),
    )

    test_cases = [
        ('/some/path', True),
        ('/some/path/inner', True),
        ('/some/path/but', True),
        ('/some/path/but/excluded', False),
        ('/some/other/path', False),
    ]

    for path, should_authenticate in test_cases:
        # Reset and prepare scope
        scope = mock_scope.copy()
        scope['path'] = path
        scope['raw_path'] = path.encode()

        # Clear previous values
        scope.pop('user', None)
        scope.pop('auth', None)

        await middleware(scope, mock_receive, mock_send)

        if should_authenticate:
            assert scope.get('user') == 'the-user', f'user not set for {path}'
            assert scope.get('auth') == 'the-auth', f'auth not set for {path}'
        else:
            assert 'user' not in scope, f'user should not be set for {path}'
            assert 'auth' not in scope, f'auth should not be set for {path}'



import argparse
import re
import subprocess
from pathlib import Path
from typing import Literal

from inquirer import List, prompt
from inquirer.shortcuts import confirm

rc_base_questions = [
    List(
        'rc_base',
        message='Which version is this a release candidate for?',
        choices=['patch', 'minor', 'major'],
    )
]

rc_next_step_questions = [
    List(
        'rc_next',
        message='You are already on a release candidate. What would you like to do?',
        choices=['Bump to next rc', 'Promote to final release'],
    )
]

main_bump_questions = [
    List(
        'bump_type',
        message='What kind of release are you preparing?',
        choices=['rc', 'patch', 'minor', 'major'],
    )
]


def bumpit(
    from_version: str,
    rule: Literal['major', 'minor', 'patch', 'rc'],
    rc_base: str | None = None,
    promote_rc: bool = False,
) -> tuple[str, str]:
    if '-' in from_version:
        main_version, pre_release = from_version.split('-')
    else:
        main_version = from_version
        pre_release = None

    major, minor, patch = map(int, main_version.split('.'))

    if rule == 'major':
        major += 1
        minor = 0
        patch = 0
        pre_release = None
    elif rule == 'minor':
        minor += 1
        patch = 0
        pre_release = None
    elif rule == 'patch':
        patch += 1
        pre_release = None
    elif rule == 'rc':
        if pre_release and pre_release.startswith('rc'):
            if promote_rc:
                pre_release = None
            else:
                rc_number = int(pre_release[2:]) + 1
                pre_release = f'rc{rc_number}'
        else:
            if rc_base == 'major':
                major += 1
                minor = 0
                patch = 0
            elif rc_base == 'minor':
                minor += 1
                patch = 0
            elif rc_base == 'patch':
                patch += 1
            else:
                raise ValueError("rc_base must be 'major', 'minor', or 'patch'")
            pre_release = 'rc1'
    else:
        raise ValueError("Invalid rule. Use 'major', 'minor', 'patch', or 'rc'.")

    to_version = f'{major}.{minor}.{patch}'
    if pre_release:
        to_version += f'-{pre_release}'

    return from_version, to_version


def bump_version(pyproject_file: Path) -> None:
    """Bump the version of a pyproject.toml file"""
    try:
        with open(pyproject_file, 'r') as f:
            pyproject = f.read()

        version_match = re.search(r'^\s*version\s*=\s*["\'](.+?)["\']', pyproject, re.MULTILINE)
        if not version_match:
            print('Could not find a valid version line.')
            raise SystemExit(1)
        version_line = version_match.group(0)
        version = version_match.group(1)
        from_version = version.split('+')[0]

        bump_type = None
        rc_base = None
        promote_rc = False

        if '-' in from_version and from_version.split('-')[1].startswith('rc'):
            rc_next_result = prompt(rc_next_step_questions)
            if not rc_next_result:
                print('Cancelled.')
                raise SystemExit(1)
            bump_type = 'rc'
            if rc_next_result['rc_next'] == 'Promote to final release':
                promote_rc = True
        else:
            result = prompt(main_bump_questions)
            if not result:
                print('No version bump requested')
                raise SystemExit(1)

            bump_type = result['bump_type']

            if bump_type == 'rc':
                rc_base_result = prompt(rc_base_questions)
                if not rc_base_result:
                    print('Cancelled.')
                    raise SystemExit(1)
                rc_base = rc_base_result['rc_base']

        from_version, to_version = bumpit(from_version, bump_type, rc_base, promote_rc)

        print(f'Bumping version from {from_version} to {to_version}')

        proceed = confirm(
            message=f'Is it correct? {from_version} -> {to_version}',
            default=True,
        )

        if proceed is True:
            with open(pyproject_file, 'w', newline='\n') as f:
                pyproject = pyproject.replace(version_line, f'version = "{to_version}"')
                f.write(pyproject)
            print(f'Bumped version from {from_version} to {to_version}')

            # uv sync --all-packages --all-extras --all-groups --upgrade
            print('Running uv sync --all-packages --all-extras --all-groups --upgrade')
            subprocess.run(
                ['uv', 'sync', '--all-packages', '--all-extras', '--all-groups', '--upgrade'],
                check=True,
            )

            # commit?
            commit_changes = confirm(message='Do you want to commit the changes?', default=True)
            if commit_changes:
                try:
                    # Commit the changes using git
                    subprocess.run(['git', 'add', '.'], check=True)
                    subprocess.run(['git', 'commit', '-m', f'release: 🔖 {to_version}'], check=True)
                    print('Changes committed successfully.')
                except subprocess.CalledProcessError as e:
                    print(f'Error committing changes: {e}')
        else:
            print('Cancelled.')
            raise SystemExit(1)

    except KeyboardInterrupt:
        print('\nCancelled.')
        raise SystemExit(1)


if __name__ == '__main__':
    parser = argparse.ArgumentParser(
        description='Bump the version of a pyproject.toml file interactively',
    )
    parser.add_argument(
        'path',
        type=Path,
        nargs='?',
        default=Path.cwd() / 'pyproject.toml',
        help='Path to the pyproject.toml file (defaults to the current directory)',
    )
    args = parser.parse_args()

    bump_version(args.path)



import ast
import io
import os
import pathlib
import subprocess
import sys
from inspect import getsource
from typing import List, Tuple

from litestar.config.app import AppConfig

decorators_dicts_path = os.path.join(
    os.path.dirname(__file__), '..', 'src/satlite/application/config'
)

dataclasses = [
    (AppConfig, decorators_dicts_path, 'LitestarAppConfigDict', 'litestar'),
]


if isinstance(sys.stdout, io.TextIOWrapper) and sys.version_info >= (3, 7):
    sys.stdout.reconfigure(encoding='utf-8')


def to_snake_case(name: str) -> str:
    return ''.join(['_' + char.lower() if char.isupper() else char for char in name]).lstrip('_')


def extract_imports(code: str) -> List[Tuple[str, List[str]]]:
    tree = ast.parse(code)
    imports = []

    for node in ast.walk(tree):
        if isinstance(node, ast.Import):
            for alias in node.names:
                imports.append((alias.name, None))
        elif isinstance(node, ast.ImportFrom):
            module = '.' * node.level + (node.module or '')
            comma_sep_names = [alias.name for alias in node.names]

            imports.append((module, comma_sep_names))

    # extra imports
    imports.append(('litestar.config.app', ['ExperimentalFeatures']))

    return imports


def fix_rel_import(import_line: str, module_path: str, destination: str) -> str:
    if not import_line.startswith('.'):
        return import_line

    module_dir = pathlib.Path(module_path).parent
    new_path = pathlib.Path(destination)

    def import_path_to_os_relative_path(import_path: str) -> str:
        parts = import_path.lstrip('.').split('.')
        parent_dirs = ['..'] * (len(import_path) - len(import_path.lstrip('.')) - 1)
        return f"./{'/'.join(parent_dirs + parts)}.py"

    relative_import_path = import_path_to_os_relative_path(import_line)
    absolute_import_path = os.path.abspath(os.path.join(module_dir, relative_import_path))

    relative_path = pathlib.Path(os.path.relpath(absolute_import_path, new_path))
    relative_parts = list(
        map(
            lambda part: '' if part == '..' else (str(part).replace('.py', '')), relative_path.parts
        )
    )

    return f"{'.'.join(relative_parts)}"


def extract_class_vars(code: str, class_name: str) -> List[Tuple[str, str, str | None]]:
    class ClassVisitor(ast.NodeVisitor):
        def __init__(self) -> None:
            self.variables: List[Tuple[str, str, str | None]] = []

        def visit_ClassDef(self, node: ast.ClassDef) -> None:
            if node.name == class_name:
                for item in node.body:
                    if isinstance(item, (ast.Assign, ast.AnnAssign)):
                        targets = item.targets if isinstance(item, ast.Assign) else [item.target]
                        for target in targets:
                            if isinstance(target, ast.Name):
                                var_name = target.id
                                var_type = ast.get_source_segment(
                                    code,
                                    item.value if isinstance(item, ast.Assign) else item.annotation,
                                )

                                if var_name is not None and var_type is not None:
                                    self.variables.append((var_name, var_type, None))

                    elif isinstance(item, ast.Expr) and isinstance(item.value, ast.Constant):
                        # docstring
                        # get last variable
                        if len(self.variables) > 0:
                            var = self.variables[-1]
                            self.variables[-1] = (var[0], var[1], item.value.value)

    visitor = ClassVisitor()
    visitor.visit(ast.parse(code))

    return visitor.variables


def extract_type_aliases(code: str) -> List[str]:
    type_aliases = []

    def visit_node(node: ast.AST) -> None:
        if isinstance(node, ast.AnnAssign):
            target, annotation = node.target, node.annotation
            if (
                isinstance(target, ast.Name)
                and isinstance(annotation, ast.Name)
                and annotation.id == 'TypeAlias'
            ):
                type_aliases.append(target.id)

    tree = ast.parse(code)
    for node in ast.walk(tree):
        visit_node(node)

    return type_aliases


def relcwd(path: str) -> str:
    return os.path.relpath(path, os.getcwd()).replace('\\', '/')


def reformat_docstring(docstring: str, max_line_length: int = 90) -> str:
    # Remove all existing newlines and normalize whitespace
    docstring = docstring.replace('\n', ' ')
    docstring = ' '.join(docstring.split())

    # Break the text into lines of max_line_length
    lines = []
    while len(docstring) > max_line_length:
        # Find the last space within the max_line_length
        split_at = docstring.rfind(' ', 0, max_line_length + 1)
        if split_at == -1:
            split_at = max_line_length
        lines.append(docstring[:split_at])
        docstring = docstring[split_at:].lstrip()

    if docstring:
        lines.append(docstring)

    # Apply indentation to all but the first line
    if len(lines) > 1:
        lines = [lines[0]] + ['    ' + line for line in lines[1:]]

    return '\n'.join(lines)


def write_file(
    typed_dict_path: str,
    module_path: str,
    module_src: str,
    dataclass_name: str,
    new_name: str | None = None,
) -> None:
    relative_module_path_from_cwd = relcwd(module_path).replace('\\', '/')

    with open(typed_dict_path, 'w', encoding='utf8') as f:
        # docstring at the top of the file indicating that it is auto-generated by the tool
        f.writelines(
            [
                '"""\n',
                f'Module providing typed dicts for the dataclass {dataclass_name}:\n',
                f'- {relative_module_path_from_cwd}\n\n',
                'ATTENTION:\n',
                'This file was auto-generated by `task gen:dicts`.\n',
                'Do not edit manually.\n',
                '"""\n\n',
            ]
        )

        f.write('from typing import TypedDict\n')
        # imports
        for module, names in extract_imports(module_src):
            if names is None:
                f.write(f'import {module}\n')
            else:
                module = fix_rel_import(module, module_path, typed_dict_path)
                f.write(f'from {module} import {", ".join(names)}\n')

            # import type aliases
        for alias in extract_type_aliases(module_src):
            # type aliases are defined in the same module as the class, so get the file name
            # of the module (without the .py extension) and import the alias from there
            module_name = fix_rel_import(
                f'.{pathlib.Path(module_path).stem}', module_path, typed_dict_path
            )
            f.write(f'from {module_name} import {alias}\n')

            # class as TypedDict
        f.write(f'class {new_name or dataclass_name}(TypedDict, total=False):\n')
        for var_name, var_type, docstring in extract_class_vars(module_src, dataclass_name):
            f.write(f'    {var_name}: {var_type}\n')  # noqa: S603

            if docstring is not None:
                f.write(f'    """{reformat_docstring(docstring)}"""\n\n')


def types() -> None:
    print('❯ Generating TypeDict for Objects:\n')

    for dc, types_path, dict_name, file_name in dataclasses:
        # create the types directory if it doesn't exist
        pathlib.Path(types_path).mkdir(parents=True, exist_ok=True)

        dc_name = str(dc.__name__)
        type_dict_class_name = dict_name or f'{dc_name}Dict'

        file_name = file_name or to_snake_case(dc_name)

        print(f'📑 Generating {type_dict_class_name} from {dc_name} in {relcwd(types_path)}')

        typed_dict_path = os.path.join(types_path, f'{to_snake_case(file_name)}.py')

        module = sys.modules[dc.__module__]
        module_path = str(module.__file__)
        module_src = getsource(module)

        write_file(typed_dict_path, module_path, module_src, dc_name, type_dict_class_name)
        print('    ⚬ The module was generated')

        # lint the new file using ruff
        poetry_commands = [
            ('formatting', 'uv', f'run ruff format {typed_dict_path}'),
            ('linting', 'uv', f'run ruff check {typed_dict_path} --fix'),
        ]

        for action, command, args in poetry_commands:
            print(f'{" " * 4}⚬ $ {command} {args}')
            try:
                subprocess.run(f'{command} {args}', check=True, capture_output=True)  # noqa: S603
            except subprocess.CalledProcessError as e:
                msg = ' | '.join(str(e.output.decode('utf-8')).rstrip('\n').split('\n'))
                print(
                    f'{" " * 4}Err '
                    f'while {action} {relcwd(typed_dict_path)} '
                    f'with ruff\n\n'
                    f'{" " * 4}{msg}'
                )

        print()

    print(
        '\n❯ `TypeDict` generation completed\n',
    )


if __name__ == '__main__':
    types()



from litestar.datastructures import MutableScopeHeaders
from litestar.enums import ScopeType
from litestar.middleware import ASGIMiddleware
from litestar.types import ASGIApp, Message, Receive, Scope, Send


class ServerNameMiddleware(ASGIMiddleware):
    """Middleware to add the server name to the response headers."""

    scopes = (ScopeType.HTTP, ScopeType.ASGI)

    def __init__(self, server_name: str = 'Satlite') -> None:
        super().__init__()
        self.server_name = server_name

    async def handle(self, scope: Scope, receive: Receive, send: Send, next_app: ASGIApp) -> None:
        async def send_wrapper(message: Message) -> None:
            # set the server name header in all responses
            if message['type'] == 'http.response.start':
                headers = MutableScopeHeaders.from_message(message=message)
                headers['Server'] = self.server_name
            await send(message)

        await next_app(scope, receive, send_wrapper)



from typing import TYPE_CHECKING

from litestar_vite import ViteConfig

if TYPE_CHECKING:
    from ..settings import Vite as ViteSettings


def default_vite(
    vite_settings: 'ViteSettings',
) -> ViteConfig:
    return ViteConfig(
        asset_url=vite_settings.asset_url,
        build_command=vite_settings.build_command,
        build_watch_command=vite_settings.build_watch_command,
        bundle_dir=vite_settings.bundle_dir,
        detect_nodeenv=vite_settings.detect_nodeenv,
        dev_mode=vite_settings.dev_mode,
        host=vite_settings.host,
        hot_file=vite_settings.hot_file,
        hot_reload=vite_settings.hot_reload,
        install_command=vite_settings.install_command,
        is_react=vite_settings.is_react,
        manifest_name=vite_settings.manifest_name,
        port=vite_settings.port,
        protocol=vite_settings.protocol,
        public_dir=vite_settings.public_dir,
        resource_dir=vite_settings.resource_dir,
        root_dir=vite_settings.root_dir,
        run_command=vite_settings.run_command,
        set_environment=vite_settings.set_environment,
        set_static_folders=vite_settings.set_static_folders,
        ssr_enabled=vite_settings.ssr_enabled,
        ssr_output_dir=vite_settings.ssr_output_dir,
        use_server_lifespan=vite_settings.use_server_lifespan,
    )



"""
Module providing typed dicts for the dataclass AppConfig:
- .venv/Lib/site-packages/litestar/config/app.py

ATTENTION:
This file was auto-generated by `task gen:dicts`.
Do not edit manually.
"""

from __future__ import annotations

from contextlib import AbstractAsyncContextManager
from typing import Any, Callable, TypedDict

from litestar import Litestar, Response
from litestar.config.allowed_hosts import AllowedHostsConfig
from litestar.config.app import ExperimentalFeatures
from litestar.config.compression import CompressionConfig
from litestar.config.cors import CORSConfig
from litestar.config.csrf import CSRFConfig
from litestar.config.response_cache import ResponseCacheConfig
from litestar.connection import Request, WebSocket
from litestar.datastructures import CacheControlHeader, ETag, State
from litestar.di import Provide
from litestar.dto import AbstractDTO
from litestar.events.emitter import BaseEventEmitterBackend
from litestar.events.listener import EventListener
from litestar.logging.config import BaseLoggingConfig
from litestar.openapi.config import OpenAPIConfig
from litestar.openapi.spec import SecurityRequirement
from litestar.plugins import PluginProtocol
from litestar.static_files.config import StaticFilesConfig
from litestar.stores.base import Store
from litestar.stores.registry import StoreRegistry
from litestar.types import (
    AfterExceptionHookHandler,
    AfterRequestHookHandler,
    AfterResponseHookHandler,
    AnyCallable,
    BeforeMessageSendHookHandler,
    BeforeRequestHookHandler,
    ControllerRouterHandler,
    Debugger,
    ExceptionHandlersMap,
    Guard,
    Middleware,
    ParametersMap,
    ResponseCookies,
    ResponseHeaders,
    TypeEncodersMap,
)
from litestar.types.callable_types import LifespanHook
from litestar.types.composite_types import TypeDecodersSequence
from litestar.types.empty import EmptyType
from litestar.types.internal_types import TemplateConfigType


class LitestarAppConfigDict(TypedDict, total=False):
    after_exception: list[AfterExceptionHookHandler]
    '''An application level :class:`exception hook handler <.types.AfterExceptionHookHandler>` or
    list thereof. This hook is called after an exception occurs. In difference to exception
    handlers, it is not meant to return a response - only to process the exception (e.g. log
    it, send it to Sentry etc.).'''

    after_request: AfterRequestHookHandler | None
    '''A sync or async function executed after the route handler function returned and the
    response object has been resolved. Receives the response object which may be any subclass
    of :class:`Response <.response.Response>`.'''

    after_response: AfterResponseHookHandler | None
    '''A sync or async function called after the response has been awaited. It receives the
    :class:`Request <.connection.Request>` object and should not return any values.'''

    allowed_hosts: list[str] | AllowedHostsConfig | None
    '''If set enables the builtin allowed hosts middleware.'''

    before_request: BeforeRequestHookHandler | None
    '''A sync or async function called immediately before calling the route handler. Receives the
    :class:`Request <.connection.Request>` instance and any non-``None`` return value is used
    for the response, bypassing the route handler.'''

    before_send: list[BeforeMessageSendHookHandler]
    '''An application level :class:`before send hook handler
    <.types.BeforeMessageSendHookHandler>` or list thereof. This hook is called when the ASGI
    send function is called.'''

    cache_control: CacheControlHeader | None
    '''A ``cache-control`` header of type :class:`CacheControlHeader
    <.datastructures.CacheControlHeader>` to add to route handlers of this app. Can be
    overridden by route handlers.'''

    compression_config: CompressionConfig | None
    '''Configures compression behaviour of the application, this enabled a builtin or user
    defined Compression middleware.'''

    cors_config: CORSConfig | None
    '''If set this enables the builtin CORS middleware.'''

    csrf_config: CSRFConfig | None
    '''If set this enables the builtin CSRF middleware.'''

    debug: bool
    '''If ``True``, app errors rendered as HTML with a stack trace.'''

    dependencies: dict[str, Provide | AnyCallable]
    '''A string keyed dictionary of dependency :class:`Provider <.di.Provide>` instances.'''

    dto: type[AbstractDTO] | None | EmptyType
    ''':class:`AbstractDTO <.dto.base_dto.AbstractDTO>` to use for (de)serializing and validation
    of request data.'''

    etag: ETag | None
    '''An ``etag`` header of type :class:`ETag <.datastructures.ETag>` to add to route handlers
    of this app. Can be overridden by route handlers.'''

    event_emitter_backend: type[BaseEventEmitterBackend]
    '''A subclass of :class:`BaseEventEmitterBackend <.events.emitter.BaseEventEmitterBackend>`.'''

    exception_handlers: ExceptionHandlersMap
    '''A dictionary that maps handler functions to status codes and/or exception types.'''

    guards: list[Guard]
    '''A list of :class:`Guard <.types.Guard>` callables.'''

    include_in_schema: bool | EmptyType
    '''A boolean flag dictating whether the route handler should be documented in the OpenAPI
    schema'''

    lifespan: list[Callable[[Litestar], AbstractAsyncContextManager] | AbstractAsyncContextManager]
    '''A list of callables returning async context managers, wrapping the lifespan of the ASGI
    application'''

    listeners: list[EventListener]
    '''A list of :class:`EventListener <.events.listener.EventListener>`.'''

    logging_config: BaseLoggingConfig | None
    '''An instance of :class:`BaseLoggingConfig <.logging.config.BaseLoggingConfig>` subclass.'''

    middleware: list[Middleware]
    '''A list of :class:`Middleware <.types.Middleware>`.'''

    on_shutdown: list[LifespanHook]
    '''A list of :class:`LifespanHook <.types.LifespanHook>` called during application shutdown.'''

    on_startup: list[LifespanHook]
    '''A list of :class:`LifespanHook <.types.LifespanHook>` called during application startup.'''

    openapi_config: OpenAPIConfig | None
    '''Defaults to :data:`DEFAULT_OPENAPI_CONFIG <litestar.app.DEFAULT_OPENAPI_CONFIG>`'''

    opt: dict[str, Any]
    '''A string keyed dictionary of arbitrary values that can be accessed in :class:`Guards
    <.types.Guard>` or wherever you have access to :class:`Request <.connection.Request>` or
    :class:`ASGI Scope <litestar.types.Scope>`. Can be overridden by routers and router
    handlers.'''

    parameters: ParametersMap
    '''A mapping of :class:`Parameter <.params.Parameter>` definitions available to all
    application paths.'''

    path: str
    '''A base path that prefixed to all route handlers, controllers and routers associated with
    the application instance. .. versionadded:: 2.8.0'''

    pdb_on_exception: bool
    '''Drop into the PDB on an exception'''

    debugger_module: Debugger
    '''A `pdb`-like debugger module that supports the `post_mortem()` protocol. This module will
    be used when `pdb_on_exception` is set to True.'''

    plugins: list[PluginProtocol]
    '''List of :class:`SerializationPluginProtocol <.plugins.SerializationPluginProtocol>`.'''

    request_class: type[Request] | None
    '''An optional subclass of :class:`Request <.connection.Request>` to use for http
    connections.'''

    request_max_body_size: int | None | EmptyType
    '''Maximum allowed size of the request body in bytes. If this size is exceeded, a '413 -
    Request Entity Too Large' error response is returned.'''

    response_class: type[Response] | None
    '''A custom subclass of :class:`Response <.response.Response>` to be used as the app's
    default response.'''

    response_cookies: ResponseCookies
    '''A list of :class:`Cookie <.datastructures.Cookie>`.'''

    response_headers: ResponseHeaders
    '''A string keyed dictionary mapping :class:`ResponseHeader
    <.datastructures.ResponseHeader>`.'''

    response_cache_config: ResponseCacheConfig
    '''Configures caching behavior of the application.'''

    return_dto: type[AbstractDTO] | None | EmptyType
    ''':class:`AbstractDTO <.dto.base_dto.AbstractDTO>` to use for serializing outbound response
    data.'''

    route_handlers: list[ControllerRouterHandler]
    '''A required list of route handlers, which can include instances of :class:`Router
    <.router.Router>`, subclasses of :class:`Controller <.controller.Controller>` or any
    function decorated by the route handler decorators.'''

    security: list[SecurityRequirement]
    '''A list of dictionaries that will be added to the schema of all route handlers in the
    application. See :data:`SecurityRequirement <.openapi.spec.SecurityRequirement>` for
    details.'''

    signature_namespace: dict[str, Any]
    '''A mapping of names to types for use in forward reference resolution during signature
    modelling.'''

    signature_types: list[Any]
    '''A sequence of types for use in forward reference resolution during signature modelling.
    These types will be added to the signature namespace using their ``__name__`` attribute.'''

    state: State
    '''A :class:`State` <.datastructures.State>` instance holding application state.'''

    static_files_config: list[StaticFilesConfig]
    '''An instance or list of :class:`StaticFilesConfig <.static_files.StaticFilesConfig>`.'''

    stores: StoreRegistry | dict[str, Store] | None
    '''Central registry of :class:`Store <.stores.base.Store>` to be made available and be used
    throughout the application. Can be either a dictionary mapping strings to :class:`Store
    <.stores.base.Store>` instances, or an instance of :class:`StoreRegistry
    <.stores.registry.StoreRegistry>`.'''

    tags: list[str]
    '''A list of string tags that will be appended to the schema of all route handlers under the
    application.'''

    template_config: TemplateConfigType | None
    '''An instance of :class:`TemplateConfig <.template.TemplateConfig>`.'''

    type_decoders: TypeDecodersSequence | None
    '''A sequence of tuples, each composed of a predicate testing for type identity and a msgspec
    hook for deserialization.'''

    type_encoders: TypeEncodersMap | None
    '''A mapping of types to callables that transform them into types supported for
    serialization.'''

    websocket_class: type[WebSocket] | None
    '''An optional subclass of :class:`WebSocket <.connection.WebSocket>` to use for websocket
    connections.'''

    multipart_form_part_limit: int
    '''The maximal number of allowed parts in a multipart/formdata request. This limit is
    intended to protect from DoS attacks.'''

    experimental_features: list[ExperimentalFeatures] | None



from __future__ import annotations

from copy import copy
from dataclasses import dataclass, field
from datetime import datetime, timedelta, timezone
from typing import Any, Callable, Generic, Iterable, Literal, Sequence, TypeVar, cast

from litestar import Controller, MediaType, Response, Router
from litestar.config.app import AppConfig
from litestar.connection import ASGIConnection
from litestar.datastructures import Cookie
from litestar.di import Provide
from litestar.handlers import HTTPRouteHandler
from litestar.middleware import DefineMiddleware
from litestar.middleware._utils import (
    build_exclude_path_pattern,
)
from litestar.openapi.spec import (
    Components,
    OAuthFlow,
    OAuthFlows,
    SecurityRequirement,
    SecurityScheme,
)
from litestar.routes import HTTPRoute
from litestar.status_codes import HTTP_201_CREATED
from litestar.types import (
    ControllerRouterHandler,
    Guard,
    Method,
    Scopes,
    SyncOrAsyncUnion,
    TypeEncodersMap,
)

from .inclusive_abstract_auth import (
    InclusiveAbstractAuthenticationMiddleware,
    build_include_path_pattern,
)

UserType = TypeVar('UserType')
TokenT = TypeVar('TokenT')
T = TypeVar('T')

__all__ = (
    'InclusiveJwtOauthCookieAuth',
    'InclusiveAbstractAuthenticationMiddleware',
    'build_include_path_pattern',
)


@dataclass
class InclusiveJwtOauthCookieAuth(Generic[UserType, TokenT]):
    """Jwt Oauth2 Cookie Authentication Configuration."""

    retrieve_user_handler: Callable[[TokenT, ASGIConnection], SyncOrAsyncUnion[UserType | None]]
    '''
    Callable that receives the `auth` value from the authentication middleware and returns a
    `User` value.
    '''

    token_url: str
    '''The URL for retrieving a new token.'''

    middleware_class: type[InclusiveAbstractAuthenticationMiddleware]
    '''The middleware class to use.'''

    token_cls: type[TokenT]
    '''The class to use for the token.'''

    oauth_scopes: dict[str, str] | None = field(default=None)
    '''Oauth Scopes available for the token.'''

    revoked_token_handler: Callable[[Any, ASGIConnection], SyncOrAsyncUnion[bool]] | None = field(
        default=None
    )
    '''
    Callable that receives the auth value from the authentication middleware and checks whether the
    token has been revoked, returning `True` if revoked, `False` otherwise.

    **Note**: <i>Not Implemented core-side yet.</i>
    '''

    guards: Iterable[Guard] | None = field(default=None)
    '''An iterable of guards to call for requests, providing authorization functionalities.'''

    include: str | list[str] | None = field(default=None)
    '''A pattern or list of patterns to include in the authentication middleware.'''

    exclude: str | list[str] | None = field(default=None)
    '''A pattern or list of patterns to skip in the authentication middleware.'''

    exclude_from_auth_key: str = field(default='public')
    '''A key to use on routes to disable authentication checks for a particular route.'''

    scopes: Scopes | None = field(default=None)
    '''
    ASGI scopes processed by the authentication middleware, if `None`, both `http` and `websocket`
    will be processed.
    '''

    exclude_http_methods: Sequence[Method] | None = field(
        default_factory=lambda: cast('Sequence[Method]', ['OPTIONS', 'HEAD'])
    )
    '''Http methods that don't require auth. Defaults to `['OPTIONS', 'HEAD']`'''

    route_handlers: Iterable[ControllerRouterHandler] | None = field(default=None)
    '''An optional iterable of route handlers to register.'''

    dependencies: dict[str, Provide] | None = field(default=None)
    '''An optional dictionary of dependency providers.'''

    type_encoders: TypeEncodersMap | None = field(default=None)
    '''A map of types to callables that transform them into types supported for serialization.'''

    auth_header: str = field(default='Authorization')
    '''Request header key from which to retrieve the token. E.g. `Authorization` or `X-Api-Key`.'''

    openapi_security_scheme_name: str = field(default='BearerToken')
    '''The value to use for the OpenAPI security scheme and security requirements.'''

    key: str = field(default='token')
    '''Key for the cookie.'''

    path: str = field(default='/')
    '''Path that must exist in the request url for the cookie to be valid. Defaults to `/`.'''

    domain: str | None = field(default=None)
    '''Domain for which the cookie is valid.'''

    secure: bool | None = field(default=None)
    '''Https is required for the cookie.'''

    scheme: str | None = field(default='Bearer')
    '''The scheme to use for the OpenAPI security scheme. E.g. `Bearer` or `Token`.'''

    samesite: Literal['lax', 'strict', 'none'] = field(default='lax')
    '''Controls whether or not a cookie is sent with cross-site requests. Defaults to `lax`. '''

    description: str = field(default='API Authentication')
    '''Description for the OpenAPI security scheme.'''

    @property
    def openapi_components(self) -> Components:
        """Create OpenAPI documentation for the JWT Cookie auth scheme."""
        return Components(
            security_schemes={
                self.openapi_security_scheme_name: SecurityScheme(
                    type='oauth2',
                    scheme=self.scheme,
                    name=self.auth_header,
                    security_scheme_in='header',
                    flows=OAuthFlows(password=self.oauth_flow),
                    bearer_format='JWT',
                    description=self.description,
                )
            }
        )

    @property
    def middleware(self) -> DefineMiddleware:
        """Create a middleware wrapped in `DefineMiddleware`."""
        return DefineMiddleware(
            self.middleware_class,
            auth_header=self.auth_header,
            auth_cookie_key=self.key,
            exclude=self.exclude,
            exclude_http_methods=self.exclude_http_methods,
            exclude_from_auth_key=self.exclude_from_auth_key,
            include=self.include,
            retrieve_user_handler=self.retrieve_user_handler,
            scopes=self.scopes,
            token_cls=self.token_cls,
            revoked_token_handler=self.revoked_token_handler,
        )

    @property
    def security_requirement(self) -> SecurityRequirement:
        """Return OpenAPI 3.1 security requirement schemes"""
        return {self.openapi_security_scheme_name: []}

    @property
    def oauth_flow(self) -> OAuthFlow:
        """Create an OpenAPI OAuth2 flow for the password bearer authentication scheme."""
        return OAuthFlow(token_url=self.token_url, scopes=self.oauth_scopes)

    async def create_response(
        self,
        body: T,
        token: str,
        *,
        exp: datetime | None = None,
        response_media_type: str | MediaType = MediaType.JSON,
        response_status_code: int = HTTP_201_CREATED,
    ) -> Response[T]:
        """Create a response with a JWT header and cookie."""
        if exp is None:
            # hardcode to 2 hours from now
            exp = datetime.now(timezone.utc) + timedelta(hours=2)

        return Response(
            content=body,
            status_code=response_status_code,
            media_type=response_media_type,
            headers={self.auth_header: self.format_auth_header(token)},
            cookies=[
                Cookie(
                    key=self.key,
                    path=self.path,
                    httponly=True,
                    value=self.format_auth_header(token),
                    max_age=int((exp - datetime.now(timezone.utc)).total_seconds()),
                    secure=self.secure,
                    samesite=self.samesite,
                    domain=self.domain,
                )
            ],
            type_encoders=self.type_encoders,
        )

    def format_auth_header(self, encoded_token: str) -> str:
        """Format a token according to the specified OpenAPI scheme."""
        return f'{self.scheme} {encoded_token}' if self.scheme else encoded_token

    def on_app_init(self, app_config: AppConfig) -> AppConfig:
        """Add auth security schemes to the OpenApi config."""
        app_config.middleware.insert(0, self.middleware)

        if app_config.openapi_config:
            app_config.openapi_config = copy(app_config.openapi_config)
            if isinstance(app_config.openapi_config.components, list):
                app_config.openapi_config.components.append(self.openapi_components)
            else:
                app_config.openapi_config.components = [
                    self.openapi_components,
                    app_config.openapi_config.components,
                ]

            def set_security(handler: HTTPRouteHandler, path: str | None = None) -> None:
                # set security for the handler, except for the ones that are excluded
                if handler.opt.get(self.exclude_from_auth_key, False) is True:
                    return

                include_re, exclude_re = (
                    build_include_path_pattern(include=self.include) if self.include else None,
                    build_exclude_path_pattern(exclude=self.exclude) if self.exclude else None,
                )

                if include_re and path and not include_re.match(path):
                    return

                if exclude_re and path and exclude_re.match(path):
                    return

                if not handler.security:
                    handler.security = [self.security_requirement]
                else:
                    if isinstance(handler.security, list):
                        handler.security.append(self.security_requirement)
                    else:
                        handler.security = [self.security_requirement]

            for route in app_config.route_handlers:
                if isinstance(route, type) and issubclass(route, Controller):
                    # for each function in the Controller class, print the function name
                    for attr_name in dir(route):
                        attr = getattr(route, attr_name)
                        if isinstance(attr, HTTPRouteHandler):
                            set_security(attr, route.path)
                elif isinstance(route, HTTPRouteHandler):
                    set_security(route)
                elif isinstance(route, Router):
                    for child_route in route.routes:
                        if isinstance(child_route, HTTPRoute):
                            for handler in child_route.route_handlers:
                                if isinstance(handler, HTTPRouteHandler):
                                    set_security(handler)

        if self.guards:
            app_config.guards.extend(self.guards)

        if self.dependencies:
            app_config.dependencies.update(self.dependencies)

        if self.route_handlers:
            app_config.route_handlers.extend(self.route_handlers)

        if self.type_encoders is None:
            self.type_encoders = app_config.type_encoders

        return app_config



"""
This module contains the logic for loading application settings from environment variables.
It uses the `typed-settings` library to define settings classes and load them from the environment.

More information about `typed-settings` can be found in the documentation:
https://typed-settings.readthedocs.io
"""

from functools import lru_cache, wraps
from pathlib import Path
from typing import Any, Sequence, TypeVar

try:
    import typed_settings as ts
    from dotenv import load_dotenv
    from typed_settings.processors import Processor
except ImportError:
    raise ImportError(
        '`typed-settings` and `python-dotenv` are required for this module to work. '
        'Install extra dependencies with `pip install satlite[typed-settings]` or install them '
        'manually with `pip install typed-settings python-dotenv`.'
    )

T = TypeVar('T')


class DotenvLoader(ts.EnvLoader):
    """Loader for environment variables from a `.env` file."""

    def __init__(self, prefix: str, dotenv_path: str | None = None, override: bool = False) -> None:
        load_dotenv(dotenv_path, override=override)
        super().__init__(prefix=f'{prefix.upper()}_'.replace('-', '_'))


class PydanticFieldConverter(ts.converters.TSConverter):
    """Converter for Pydantic fields. This is a workaround for the fact that `typed-settings` does
    not support Pydantic fields natively. It uses the `FieldInfo` class from Pydantic to get the
    default value of a field.
    """

    def structure(self, value: Any, cls: type[Any]) -> Any:
        try:
            from pydantic.fields import FieldInfo

            if isinstance(value, FieldInfo):
                value = value.get_default(call_default_factory=True)
        except ImportError:
            # In case pydantic is not installed, we can ignore this
            pass
        return super().structure(value, cls)


def get_settings(
    settings_type: type[T],
    *,
    prefix: str,
    strlist_sep: str = '|',
    dotenv_path: str | Path | None = None,
    override_envvars: bool = False,
    base_dir: Path = Path(),
    processors: Sequence[Processor] = (),
) -> T:
    """
    Load settings from environment variables. **This function is cached and will only be called
    once.**
    """

    if isinstance(dotenv_path, Path):
        dotenv_path = str(dotenv_path)

    return ts.load_settings(
        settings_type,
        loaders=[DotenvLoader(prefix=prefix, dotenv_path=dotenv_path, override=override_envvars)],
        converter=PydanticFieldConverter(strlist_sep=strlist_sep),
        base_dir=base_dir,
        processors=processors,
    )


# workaround for https://github.com/python/typeshed/issues/11280 (lru_cache changes the signature of
# the function, making typings incorrect)
get_settings = wraps(get_settings)(lru_cache(maxsize=1, typed=True)(get_settings))



from dataclasses import fields, is_dataclass
from typing import Any, Callable, Dict, List, Set, Tuple, Union

try:
    import structlog.dev as structlog_dev
except ImportError:
    structlog_dev = None


def c(name: str) -> Callable[[str], str]:
    use_colors = structlog_dev and structlog_dev._use_colors
    if structlog_dev and structlog_dev._use_colors:
        colors = {
            'magenta': structlog_dev.MAGENTA,
            'cyan': structlog_dev.CYAN,
            'green': structlog_dev.GREEN,
            'yellow': structlog_dev.YELLOW,
            'red': structlog_dev.RED,
            'blue': structlog_dev.BLUE,
            'reset': structlog_dev.RESET_ALL,
        }
    else:
        colors = {}

    def noop(text: str) -> str:
        return text

    def colorize(text: str) -> str:
        return colors.get(name, '') + text + colors.get('reset', '')

    return colorize if use_colors else noop


def pretty_repr(obj: Any, indent: int = 0, top_level: bool = True) -> str:
    lines: List[str] = []

    def _add_line(text: str) -> None:
        lines.append('  ' * indent + text)

    if is_dataclass(obj):
        if top_level:
            _add_line(c('magenta')(type(obj).__name__))
        for f in fields(obj):
            value = getattr(obj, f.name)
            if is_dataclass(value):
                lines.append('  ' * (indent + 1) + f'- {c("magenta")(f.name)}')
                lines.append(pretty_repr(value, indent + 2, top_level=False))
            elif isinstance(value, dict):
                lines.append('  ' * (indent + 1) + f'- {c("magenta")(f.name)}:')
                lines.append(pretty_repr_dict(value, indent + 2))
            elif isinstance(value, (list, tuple, set)):
                lines.append('  ' * (indent + 1) + f'- {c("magenta")(f.name)}:')
                lines.append(pretty_repr_iterable(value, indent + 2))
            else:
                lines.append('  ' * (indent + 1) + f'- {c("blue")(f.name)}: {value}')
    else:
        _add_line(str(obj))

    return '\n'.join(lines)


def pretty_repr_dict(d: Dict[Any, Any], indent: int) -> str:
    lines: List[str] = []
    for key, value in d.items():
        if isinstance(value, dict):
            lines.append('  ' * indent + f'- {c("green")(key)}:')
            lines.append(pretty_repr_dict(value, indent + 2))
        elif is_dataclass(value):
            lines.append('  ' * indent + f'- {c("green")(key)}')
            lines.append(pretty_repr(value, indent + 2))
        elif isinstance(value, (list, tuple, set)):
            lines.append('  ' * indent + f'- {c("green")(key)}:')
            lines.append(pretty_repr_iterable(value, indent + 2))
        else:
            lines.append('  ' * indent + f'- {c("green")(key)}: {value}')
    return '\n'.join(lines)


def pretty_repr_iterable(it: Union[List[Any], Tuple[Any, ...], Set[Any]], indent: int) -> str:
    lines: List[str] = []

    if len(it) == 0:
        return '  ' * (indent + 1) + f'| {c("yellow")("[]")}'

    for item in it:
        if is_dataclass(item):
            lines.append(pretty_repr(item, indent))
        elif isinstance(item, dict):
            lines.append(pretty_repr_dict(item, indent))
        elif isinstance(item, (list, tuple, set)):
            lines.append(pretty_repr_iterable(item, indent))
        else:
            lines.append('  ' * (indent + 1) + f'| {c("yellow")(str(item))}')
    return '\n'.join(lines)


def pretty_print(obj: Any) -> None:
    print(pretty_repr(obj))



import os
import sys
from pathlib import Path
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from .config.settings import App as AppSettings
    from .config.settings import Server as ServerSettings


def setup_environment(
    *,
    app_location: str,
    app_settings: 'AppSettings',
    server_settings: 'ServerSettings',
) -> None:
    """Configure the environment variables and path."""
    current_path = Path(__file__).parent.parent.resolve()
    sys.path.append(str(current_path))

    # https://github.com/cofin/litestar-granian/blob/main/litestar_granian/cli.py
    os.environ.setdefault('LITESTAR_APP', app_location)
    os.environ.setdefault('LITESTAR_APP_NAME', app_settings.name)
    os.environ.setdefault('LITESTAR_HOST', server_settings.host)
    os.environ.setdefault('LITESTAR_PORT', str(server_settings.port))
    os.environ.setdefault('LITESTAR_DEBUG', str(app_settings.debug))



from __future__ import annotations

import re
from abc import abstractmethod
from typing import (
    TYPE_CHECKING,
    Any,
    Awaitable,
    Callable,
    Generic,
    Iterable,
    Pattern,
    Sequence,
    TypeVar,
)

from litestar.connection import ASGIConnection
from litestar.exceptions import ImproperlyConfiguredException
from litestar.middleware import AbstractAuthenticationMiddleware, AuthenticationResult

if TYPE_CHECKING:
    from litestar.types import ASGIApp, Method, Receive, Scope, Scopes, Send

UserType = TypeVar('UserType')
TokenT = TypeVar('TokenT')


def build_include_path_pattern(*, include: str | Iterable[str] | None = None) -> Pattern | None:
    """Build single path pattern from list of patterns to include in middleware processing."""
    if include is None:
        return None

    try:
        return (
            re.compile('|'.join(include)) if not isinstance(include, str) else re.compile(include)
        )
    except re.error as e:  # pragma: no cover
        raise ImproperlyConfiguredException(
            'Unable to compile include patterns for middleware. Please make sure you passed a valid'
            ' regular expression.'
        ) from e


class InclusiveAbstractAuthenticationMiddleware(
    Generic[UserType, TokenT], AbstractAuthenticationMiddleware
):
    """Authentication middleware

    Inherit this class to create your own inclusive authentication middleware.

    When `include` is not provided, this middleware behaves the same way as
    `AbstractAuthenticationMiddleware`. That is:
    - All routes require authentication by default.
    - Only routes matching the `exclude` pattern will NOT require authentication.

    When `include` is provided:
    - All routes are "unauthenticated" by default.
    - Only routes matching the `include` pattern will require authentication.
    - Routes matching the `exclude` will NOT require authentication.
    """

    __slots__ = (
        'include',
        'retrieve_user_handler',
        'token_cls',
        'auth_cookie_key',
        'auth_header',
        'revoked_token_handler',
        *AbstractAuthenticationMiddleware.__slots__,
    )

    def __init__(
        self,
        app: ASGIApp,
        retrieve_user_handler: Callable[[Any, ASGIConnection[Any, Any, Any, Any]], Awaitable[Any]],
        token_cls: type[TokenT],
        include: str | list[str] | None = None,
        exclude: str | list[str] | None = None,
        exclude_from_auth_key: str = 'exclude_from_auth',
        exclude_http_methods: Sequence[Method] | None = None,
        scopes: Scopes | None = None,
        auth_cookie_key: str = 'token',
        auth_header: str = 'Authorization',
        revoked_token_handler: Callable[
            [TokenT, ASGIConnection[Any, Any, Any, Any]], Awaitable[Any]
        ]
        | None = None,
    ) -> None:
        """Initialize ``InclusiveAbstrctAuthenticationMiddleware``.

        **Args**
          app: An `ASGIApp`, this value is the next ASGI handler to call in the middleware stack.
          retrieve_user_handler: A callable that retrieves the user from the token.
          token_cls: A class that represents the token.
          include: A pattern or list of patterns to include in the authentication middleware.
          exclude: A pattern or list of patterns to skip in the authentication middleware.
          exclude_from_auth_key: An identifier to use on routes to disable authentication for a
          particular route.
          exclude_http_methods: A sequence of http methods that do not require authentication.
          scopes: ASGI scopes processed by the authentication middleware.
          auth_cookie_key: The name of the cookie to use for authentication.
          auth_header: The name of the header to use for authentication.
          revoked_token_handler: A callable that handles revoked tokens.
        """
        self.include = build_include_path_pattern(include=include)
        self.retrieve_user_handler = retrieve_user_handler
        self.token_cls = token_cls
        self.auth_cookie_key = auth_cookie_key
        self.auth_header = auth_header
        self.revoked_token_handler = revoked_token_handler
        super().__init__(app, exclude, exclude_from_auth_key, exclude_http_methods, scopes)

    async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
        """ASGI callable."""
        if self.include and not self.include.match(scope['path']):
            return await self.app(scope, receive, send)
        return await super().__call__(scope, receive, send)

    @abstractmethod
    async def authenticate_request(self, connection: ASGIConnection) -> AuthenticationResult:
        """Receive the http connection and return an `AuthenticationResult`.

        **Override this method in subclasses!**

        **Args**
          connection: An `ASGIConnection <litestar.connection.ASGIConnection>` instance.

        **Raises**
          `NotAuthorizedException` | `PermissionDeniedException`: if authentication fails.

        **Returns**
          An instance of `AuthenticationResult
          <litestar.middleware.authentication.AuthenticationResult>`.
        """
        raise NotImplementedError('authenticate_request must be overridden by subclasses')



"""Plugin for converting exceptions into a problem details response."""

from __future__ import annotations

from dataclasses import dataclass, field
from typing import TYPE_CHECKING

from litestar.exceptions.http_exceptions import HTTPException
from litestar.plugins import InitPlugin
from litestar.plugins.problem_details import (
    ProblemDetailsException,
    _create_exception_handler,
    _http_exception_to_problem_detail_exception,
    _problem_details_exception_handler,
)
from litestar.status_codes import HTTP_500_INTERNAL_SERVER_ERROR

if TYPE_CHECKING:
    from litestar.config.app import AppConfig
    from litestar.plugins.problem_details import (
        ExceptionToProblemDetailMapType,
        ProblemDetailsExceptionHandlerType,
    )


@dataclass
class SatliteProblemDetailsConfig:
    """The configuration object for `SatliteExceptionHandler`"""

    exception_handler: ProblemDetailsExceptionHandlerType = _problem_details_exception_handler
    '''The exception handler used for ``ProblemdetailsException.``'''

    enable_for_all_exceptions: bool = False
    '''Flag indicating whether to convert all :exc:`Exception` into ``ProblemDetailsException.``'''

    exception_to_problem_detail_map: 'ExceptionToProblemDetailMapType' = field(default_factory=dict)
    '''A mapping to convert exceptions into ``ProblemDetailsException.``

    All exceptions provided in this will get a custom exception handler where these exceptions
    are converted into ``ProblemDetailException`` before handling them. This can be used to override
    the handler for ``HTTPException`` as well.
    '''


def _default_exception_to_problem_detail(_exc: Exception) -> ProblemDetailsException:
    return ProblemDetailsException(
        status_code=HTTP_500_INTERNAL_SERVER_ERROR,
        title='Internal Server Error',
        detail='Internal Server Error',
    )


def _exception_to_problem_detail(exc: Exception) -> ProblemDetailsException:
    if isinstance(exc, HTTPException):
        return _http_exception_to_problem_detail_exception(exc)
    return _default_exception_to_problem_detail(exc)


class SatliteExceptionHandler(InitPlugin):
    """
    A customized Problem Detail plugin to convert exceptions into problem details as per RFC 9457.

    Based on `ProblemDetailsPlugin` from Litestar, but instead of convertin only HTTP exceptions,
    it can convert all exceptions into problem details (defaulting to 500 Internal Server Error).
    """

    def __init__(self, config: SatliteProblemDetailsConfig | None = None):
        self.config = config or SatliteProblemDetailsConfig()

    def on_app_init(self, app_config: AppConfig) -> AppConfig:
        app_config.exception_handlers[ProblemDetailsException] = self.config.exception_handler

        for exc_type, conversion_fn in self.config.exception_to_problem_detail_map.items():
            app_config.exception_handlers[exc_type] = _create_exception_handler(
                conversion_fn, exc_type
            )

        if self.config.enable_for_all_exceptions:
            app_config.exception_handlers[Exception] = _create_exception_handler(
                _exception_to_problem_detail, Exception
            )

        return app_config



from typing import Callable, Unpack

from litestar.config.app import AppConfig
from litestar.plugins import InitPluginProtocol

from .config.defaults import get_default_config
from .config.litestar import LitestarAppConfigDict
from .config.settings import Api as ApiSettings
from .config.settings import App as AppSettings
from .config.settings import Server as ServerSettings
from .config.settings import Vite as ViteSettings


def merge_configs(
    left: LitestarAppConfigDict, right: LitestarAppConfigDict
) -> LitestarAppConfigDict:
    """Merge two Litestar app config dictionaries."""
    merged = left.copy()
    for key, value in right.items():
        if key in merged and isinstance(merged[key], list) and isinstance(value, list):
            merged[key].extend(value)
        elif key in merged and isinstance(merged[key], dict) and isinstance(value, dict):
            merged[key].update(value)
        else:
            merged[key] = value
    return merged


class SatlitePlugin(InitPluginProtocol):
    litestar_config: LitestarAppConfigDict
    app_settings: AppSettings
    api_settings: ApiSettings
    server_settings: ServerSettings
    on_app_init_handlers: list[Callable[[AppConfig], AppConfig]]

    def __init__(
        self,
        *,
        app_settings: AppSettings = AppSettings(),
        api_settings: ApiSettings = ApiSettings(),
        server_settings: ServerSettings = ServerSettings(),
        vite_settings: ViteSettings | None = None,
        enable_csrf: bool = False,
        on_app_init: list[Callable[[AppConfig], AppConfig]] = [],
        **litestar_config: Unpack[LitestarAppConfigDict],
    ) -> None:
        self.app_settings = app_settings
        self.api_settings = api_settings
        self.server_settings = server_settings
        self.on_app_init_handlers = on_app_init

        default_cfg = get_default_config(
            app_settings, api_settings, server_settings, vite_settings, enable_csrf
        )
        self.litestar_config = merge_configs(default_cfg, litestar_config)

    def on_app_init(self, app_config: AppConfig) -> AppConfig:
        for key, value in self.litestar_config.items():
            if value is not None:
                current_value = getattr(app_config, key, None)
                if isinstance(current_value, list) and isinstance(value, list):
                    current_value.extend(value)
                elif isinstance(current_value, dict) and isinstance(value, dict):
                    current_value.update(value)
                else:
                    setattr(app_config, key, value)

        for func in self.on_app_init_handlers:
            app_config = func(app_config)
        return app_config



from typing import Any
from uuid import UUID

from litestar.enums import RequestEncodingType
from litestar.security.jwt import OAuth2Login, Token


def get_default_signature_namespaces() -> dict[str, Any]:
    return {
        'Token': Token,
        'OAuth2Login': OAuth2Login,
        'RequestEncodingType': RequestEncodingType,
        'UUID': UUID,
        # 'Body': Body,
        # 'm': m,
        # 'UserService': UserService,
        # 'RoleService': RoleService,
        # 'TeamService': TeamService,
        # 'TeamMemberService': TeamMemberService,
        # 'UserRoleService': UserRoleService,
    }



from typing import TYPE_CHECKING

from litestar.config.compression import CompressionConfig
from litestar.logging import LoggingConfig

from ..litestar import LitestarAppConfigDict
from ..middleware.server import ServerNameMiddleware
from ..plugins.exceptions import SatliteExceptionHandler, SatliteProblemDetailsConfig
from .cache import default_cache
from .cors import default_cors
from .csrf import default_csrf
from .openapi import default_openapi
from .signature_namespaces import get_default_signature_namespaces

if TYPE_CHECKING:
    from ..settings import Api as ApiSettings
    from ..settings import App as AppSettings
    from ..settings import Server as ServerSettings
    from ..settings import Vite as ViteSettings


def get_default_config(
    app_settings: 'AppSettings',
    api_settings: 'ApiSettings',
    server_settings: 'ServerSettings',
    vite_settings: 'ViteSettings | None' = None,
    enable_csrf: bool = False,
) -> LitestarAppConfigDict:
    config = LitestarAppConfigDict(
        openapi_config=default_openapi(app_settings),
        cors_config=default_cors(api_settings),
        compression_config=CompressionConfig(backend='gzip'),
        signature_namespace=get_default_signature_namespaces(),
        logging_config=LoggingConfig(),
        response_cache_config=default_cache(api_settings, app_settings),
        plugins=[
            SatliteExceptionHandler(
                config=SatliteProblemDetailsConfig(enable_for_all_exceptions=True)
            ),
        ],
        middleware=[ServerNameMiddleware(server_name=server_settings.name)],
    )

    if enable_csrf:
        config['csrf_config'] = default_csrf(api_settings)

    if vite_settings:
        from litestar.contrib.jinja import JinjaTemplateEngine
        from litestar.template import TemplateConfig

        # add vite plugin only if vite is installed
        try:
            from litestar_vite import VitePlugin

            from .vite import default_vite

            plugins = config.get('plugins')
            if plugins is not None:
                plugins.append(VitePlugin(config=default_vite(vite_settings)))
            config['template_config'] = TemplateConfig(
                engine=JinjaTemplateEngine(directory=vite_settings.template_dir)
            )
        except ImportError:
            print(
                'Error: `litestar-vite` is not installed. Install it with the extra `vite`: '
                '`pip install satlite[dev]` or `pip install satlite[vite]`.'
            )

    # add granian plugin only if granian is installed
    try:
        from litestar_granian import GranianPlugin

        plugins = config.get('plugins')
        if plugins is not None:
            plugins.append(GranianPlugin())
    except ImportError:
        pass

    # add structlog plugin only if structlog is installed
    try:
        from litestar.plugins.structlog import StructlogPlugin

        from .structlog import default_structlog

        plugins = config.get('plugins')
        if plugins is not None:
            plugins.append(StructlogPlugin(config=default_structlog()))
    except ImportError:
        pass

    return config



from typing import TYPE_CHECKING

from litestar.openapi import OpenAPIConfig
from litestar.openapi.plugins import ScalarRenderPlugin

if TYPE_CHECKING:
    from ..settings import App as AppSettings


def default_openapi(
    app_settings: 'AppSettings',
) -> OpenAPIConfig:
    """Default CORS configuration."""
    return OpenAPIConfig(
        title=app_settings.name,
        version=app_settings.version,
        # components=[jwt_auth.openapi_components],
        # security=[jwt_auth.security_requirement],
        use_handler_docstrings=True,
        render_plugins=[ScalarRenderPlugin(version='latest')],
    )



from typing import TYPE_CHECKING

from litestar import Request
from litestar.config.response_cache import ResponseCacheConfig, default_cache_key_builder

if TYPE_CHECKING:
    from ..settings import Api as ApiSettings
    from ..settings import App as AppSettings


def default_cache(
    api_settings: 'ApiSettings',
    app_settings: 'AppSettings',
) -> ResponseCacheConfig:
    """Default CORS configuration."""

    def _cache_key_builder(request: Request) -> str:
        return f'{app_settings.slug}:{default_cache_key_builder(request)}'

    return ResponseCacheConfig(
        default_expiration=api_settings.cache_expiration,
        key_builder=_cache_key_builder,
    )
